'use strict';
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const express = require("express");
const _ = require("lodash");
const typescript_ioc_1 = require("typescript-ioc");
const typescript_rest_1 = require("typescript-rest");
const swagger = require("typescript-rest-swagger");
const users_1 = require("../../config/users");
const users_2 = require("../../service/users");
let UsersRest = class UsersRest {
    static configureAuthMiddleware(app) {
        const authenticator = typescript_ioc_1.Container.get(users_2.UserService).getAuthMiddleware();
        const manageUsers = (req, res, next) => {
            if (!req.user || !req.user.roles || (req.user.roles.indexOf('admin') < 0)) {
                return next(new typescript_rest_1.Errors.ForbidenError('Access denied'));
            }
            next();
        };
        const adminGateway = (req, res, next) => {
            if (req.path === '/users/authentication' || req.path === '/healthcheck') {
                return next();
            }
            if (!req.user || !req.user.roles || (req.user.roles.indexOf('config') < 0)) {
                return next(new typescript_rest_1.Errors.ForbidenError('Access denied'));
            }
            next();
        };
        app.use((req, res, next) => {
            if (req.path === '/users/authentication' || req.path === '/healthcheck') {
                return next();
            }
            authenticator(req, res, next);
        });
        app.get('/users', manageUsers);
        app.post('/users', manageUsers);
        app.get('/users/:login', manageUsers);
        app.put('/users/:login', manageUsers);
        app.delete('/users/:login', manageUsers);
        app.use(adminGateway);
    }
    list() {
        return this.service.list();
    }
    async addUser(user) {
        await users_1.validateUser(user);
        await this.service.create(user);
        return new typescript_rest_1.Return.NewResource(`users/${user.login}`);
    }
    async getUser(login) {
        const user = await this.service.get(login);
        if (!user) {
            throw new typescript_rest_1.Errors.NotFoundError();
        }
        return _.omit(user, 'password');
    }
    async updateUser(login, user) {
        user.login = login;
        const dbUser = await this.service.get(login);
        if (!dbUser) {
            throw new typescript_rest_1.Errors.NotFoundError(`Not Found: ${login}`);
        }
        user = _.defaultsDeep(user, dbUser);
        await users_1.validateUser(user);
        await this.service.update(user);
    }
    removeUser(login) {
        return this.service.remove(login);
    }
    async getAuthToken(login, password) {
        try {
            return await this.service.generateToken(login, password);
        }
        catch (err) {
            throw new typescript_rest_1.Errors.UnauthorizedError(err.message);
        }
    }
    changePassword(req, login, password) {
        if ((req.user.login !== login) &&
            (!req.user.roles || (req.user.roles.indexOf('admin') < 0))) {
            throw new typescript_rest_1.Errors.ForbidenError('Access denied');
        }
        return this.service.changePassword(login, password);
    }
};
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", users_2.UserService)
], UsersRest.prototype, "service", void 0);
__decorate([
    typescript_rest_1.GET,
    swagger.Security('Bearer'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], UsersRest.prototype, "list", null);
__decorate([
    typescript_rest_1.POST,
    swagger.Security('Bearer'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], UsersRest.prototype, "addUser", null);
__decorate([
    typescript_rest_1.GET,
    typescript_rest_1.Path(':login'),
    swagger.Security('Bearer'),
    __param(0, typescript_rest_1.PathParam('login')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", Promise)
], UsersRest.prototype, "getUser", null);
__decorate([
    typescript_rest_1.PUT,
    typescript_rest_1.Path(':login'),
    swagger.Security('Bearer'),
    __param(0, typescript_rest_1.PathParam('login')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, Object]),
    __metadata("design:returntype", Promise)
], UsersRest.prototype, "updateUser", null);
__decorate([
    typescript_rest_1.DELETE,
    typescript_rest_1.Path(':login'),
    swagger.Security('Bearer'),
    __param(0, typescript_rest_1.PathParam('login')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", Promise)
], UsersRest.prototype, "removeUser", null);
__decorate([
    typescript_rest_1.POST,
    typescript_rest_1.Path('/authentication'),
    __param(0, typescript_rest_1.FormParam('login')), __param(1, typescript_rest_1.FormParam('password')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String]),
    __metadata("design:returntype", Promise)
], UsersRest.prototype, "getAuthToken", null);
__decorate([
    typescript_rest_1.POST,
    typescript_rest_1.Path('/authentication/changepassword'),
    swagger.Security('Bearer'),
    __param(0, typescript_rest_1.ContextRequest),
    __param(1, typescript_rest_1.FormParam('login')),
    __param(2, typescript_rest_1.FormParam('password')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String, String]),
    __metadata("design:returntype", Promise)
], UsersRest.prototype, "changePassword", null);
UsersRest = __decorate([
    typescript_rest_1.Path('users'),
    swagger.Tags('Users')
], UsersRest);
exports.UsersRest = UsersRest;
//# sourceMappingURL=users.js.map