#!/usr/bin/env node
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_ioc_1 = require("typescript-ioc");
const configuration_1 = require("../../configuration");
const database_1 = require("../../database");
const cli_args_1 = require("./cli-args");
const cli_tool_1 = require("./cli-tool");
// tslint:disable:no-console
try {
    const config = typescript_ioc_1.Container.get(configuration_1.Configuration);
    config.on('load', () => {
        const database = typescript_ioc_1.Container.get(database_1.Database);
        new cli_tool_1.Cli(cli_args_1.configArgs).processCommand()
            .then(() => {
            database.disconnect();
        }).catch((err) => {
            handleConfigError(err);
            database.disconnect();
            process.exit(1);
        });
    });
    config.on('error', error => {
        handleConfigError(error);
    });
}
catch (e) {
    console.error(e);
}
function handleConfigError(err) {
    let errorMessage;
    if (err && err.response && err.response.body && err.response.body.error) {
        errorMessage = err.response.body.error;
    }
    else if (err && err.message) {
        errorMessage = err.message;
    }
    else if (err && err.entity) {
        errorMessage = err.entity;
    }
    else {
        errorMessage = err;
    }
    try {
        errorMessage = JSON.parse(errorMessage);
        if (errorMessage.entity) {
            errorMessage = errorMessage.entity;
        }
        else if (errorMessage.message) {
            errorMessage = errorMessage.message;
        }
    }
    catch (e) {
        // IGNORE
    }
    console.error(errorMessage);
}
//# sourceMappingURL=cli.js.map