'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("./utils");
class ApisClient {
    constructor(swaggerClient) {
        this.swaggerClient = swaggerClient;
    }
    async list(filters) {
        const response = await utils_1.invoke(this.swaggerClient.apis.APIs.APIRestList(filters));
        return utils_1.getResponseBody(response);
    }
    async addApi(api) {
        const response = await utils_1.invoke(this.swaggerClient.apis.APIs.APIRestAddApi({ api: api }));
        return utils_1.getCreatedResource(response).substring(5);
    }
    async updateApi(id, api) {
        if (!id) {
            throw new Error('Invalid API id. To update an API, you must provide a valid ID field');
        }
        api.id = id;
        const response = await utils_1.invoke(this.swaggerClient.apis.APIs.APIRestUpdateApi({ id: id, api: api }));
        utils_1.checkStatus(response, 204);
    }
    async removeApi(id) {
        if (!id) {
            throw new Error('Invalid API id. To remove an API, you must provide a valid ID');
        }
        const response = await utils_1.invoke(this.swaggerClient.apis.APIs.APIRestRemoveApi({ id: id }));
        utils_1.checkStatus(response, 204);
    }
    async getApi(id) {
        if (!id) {
            throw new Error('Invalid API id. To retrieve an API, you must provide a valid ID');
        }
        const response = await utils_1.invoke(this.swaggerClient.apis.APIs.APIRestGetApi({ id: id }));
        return utils_1.getResponseBody(response);
    }
}
exports.ApisClient = ApisClient;
//# sourceMappingURL=sdk-apis.js.map