'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const Joi = require("joi");
const errors_1 = require("./errors");
const middleware_1 = require("./middleware");
const jwtRequestExtractorSchema = Joi.object().keys({
    authHeader: Joi.string(),
    bodyField: Joi.string(),
    cookie: Joi.string(),
    header: Joi.string(),
    queryParam: Joi.string()
});
const jwtAuthenticationSchema = Joi.object().keys({
    algorithms: Joi.array().items(Joi.string()),
    audience: Joi.string(),
    extractFrom: jwtRequestExtractorSchema,
    ignoreExpiration: Joi.boolean(),
    issuer: Joi.string(),
    secretOrKey: Joi.string().required(),
    verify: middleware_1.middlewareConfigValidatorSchema
});
const basicAuthenticationSchema = Joi.object().keys({
    verify: middleware_1.middlewareConfigValidatorSchema.required()
});
const localAuthenticationSchema = Joi.object().keys({
    passwordField: Joi.string(),
    usernameField: Joi.string(),
    verify: middleware_1.middlewareConfigValidatorSchema.required()
});
exports.authenticationValidatorSchema = Joi.object().keys({
    strategy: middleware_1.middlewareConfigValidatorSchema.required()
});
exports.apiAuthenticationValidatorSchema = exports.authenticationValidatorSchema.keys({
    group: Joi.alternatives([Joi.array().items(Joi.string()), Joi.string()]),
    strategy: middleware_1.middlewareConfigValidatorSchema,
    use: Joi.string()
}).xor('strategy', 'use');
function validateLocalAuthConfig(config) {
    const result = Joi.validate(config, localAuthenticationSchema);
    if (result.error) {
        throw new errors_1.ValidationError(result.error);
    }
    else {
        return result.value;
    }
}
exports.validateLocalAuthConfig = validateLocalAuthConfig;
function validateBasicAuthConfig(config) {
    const result = Joi.validate(config, basicAuthenticationSchema);
    if (result.error) {
        throw new errors_1.ValidationError(result.error);
    }
    else {
        return result.value;
    }
}
exports.validateBasicAuthConfig = validateBasicAuthConfig;
function validateJwtAuthConfig(config) {
    const result = Joi.validate(config, jwtAuthenticationSchema);
    if (result.error) {
        throw new errors_1.ValidationError(result.error);
    }
    else {
        return result.value;
    }
}
exports.validateJwtAuthConfig = validateJwtAuthConfig;
//# sourceMappingURL=authentication.js.map