"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
var Database_1;
'use strict';
const Redis = require("ioredis");
const _ = require("lodash");
const typescript_ioc_1 = require("typescript-ioc");
const configuration_1 = require("./configuration");
let Database = Database_1 = class Database {
    constructor() {
        this.client = this.initializeRedis(this.config.database.redis);
        this.events = this.initializeRedis(this.config.database.redis);
    }
    get redisClient() {
        return this.client;
    }
    get redisEvents() {
        return this.events;
    }
    disconnect() {
        this.redisClient.disconnect();
        this.redisEvents.disconnect();
    }
    async registerGatewayVersion() {
        try {
            const packageJson = require('../package.json');
            await this.redisClient.set(Database_1.GATEWAY_VERSION_KEY, `${packageJson.version}`);
        }
        catch (err) {
            throw new Error('It was not possible to register the Tree Gateway version.');
        }
    }
    async getGatewayVersion() {
        try {
            return await this.redisClient.get(Database_1.GATEWAY_VERSION_KEY);
        }
        catch (err) {
            throw (new Error('It was not possible to retrieve the Tree Gateway version.'));
        }
    }
    async startGatewayUpdate() {
        try {
            return await this.redisClient.getset(Database_1.GATEWAY_VERSION_KEY, Database_1.UPDATING);
        }
        catch (err) {
            throw (new Error('It was not possible to retrieve the Tree Gateway version.'));
        }
    }
    initializeRedis(config) {
        let client;
        config = _.defaults(config, {
            options: {}
        });
        if (config.cluster) {
            config.cluster.forEach(node => {
                node.port = _.toSafeInteger(node.port);
                node.host = node.host;
            });
            client = new Redis.Cluster(config.cluster, {
                redisOptions: config.options,
                scaleReads: 'all'
            });
        }
        else if (config.sentinel) {
            const params = _.defaults(config.options, {
                name: config.sentinel.name,
                sentinels: config.sentinel.nodes
            });
            config.sentinel.nodes.forEach(node => {
                node.port = _.toSafeInteger(node.port);
                node.host = node.host;
            });
            client = new Redis(params);
        }
        else {
            config.standalone = _.defaults(config.standalone, {
                host: 'localhost',
                port: 6379
            });
            if (config.standalone.password) {
                config.options.password = config.standalone.password;
            }
            client = new Redis(_.toSafeInteger(config.standalone.port), config.standalone.host, config.options);
        }
        return client;
    }
};
Database.GATEWAY_VERSION_KEY = '{config}:treegateway:version';
Database.UPDATING = `0.0.0`;
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", configuration_1.Configuration)
], Database.prototype, "config", void 0);
Database = Database_1 = __decorate([
    typescript_ioc_1.Singleton,
    typescript_ioc_1.AutoWired,
    __metadata("design:paramtypes", [])
], Database);
exports.Database = Database;
//# sourceMappingURL=database.js.map