'use strict';
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs-extra-promise");
const _ = require("lodash");
const path = require("path");
const typescript_ioc_1 = require("typescript-ioc");
const util_1 = require("util");
const Winston = require("winston");
const logger_1 = require("./config/logger");
const configuration_1 = require("./configuration");
let Logger = class Logger {
    constructor() {
        this.winston = this.instantiateLogger(this.config.gateway.logger);
        this.config.on('gateway-update', (gateway) => {
            if (this.winston && this.isDebugEnabled()) {
                this.debug('The system logger is restarting...');
            }
            this.winston = this.instantiateLogger(gateway.logger);
        });
    }
    isDebugEnabled() {
        return this.level === logger_1.LogLevel.debug;
    }
    isInfoEnabled() {
        return this.level >= logger_1.LogLevel.info;
    }
    isWarnEnabled() {
        return this.level >= logger_1.LogLevel.warn;
    }
    isErrorEnabled() {
        return this.level >= logger_1.LogLevel.error;
    }
    debug(...args) {
        this.winston.debug.apply(this, arguments);
    }
    info(...args) {
        this.winston.info.apply(this, arguments);
    }
    warn(...args) {
        this.winston.warn.apply(this, arguments);
    }
    error(...args) {
        this.winston.error.apply(this, arguments);
    }
    inspectObject(object) {
        util_1.inspect(object, { colors: true, depth: 15 });
    }
    instantiateLogger(config) {
        this.level = (config ? logger_1.LogLevel[config.level] : logger_1.LogLevel.info);
        const options = {
            level: logger_1.LogLevel[this.level],
            transports: []
        };
        if (config && config.console) {
            options.transports.push(new Winston.transports.Console(config.console));
        }
        if (config && config.file) {
            config.file = _.omit(config.file, 'outputDir');
            let outputDir = config.file.outputDir || './logs';
            if (_.startsWith(outputDir, '.')) {
                outputDir = path.join(this.config.rootPath, outputDir);
            }
            const fileName = (process.env.processNumber ? `gateway-${process.env.processNumber}.log` : `gateway.log`);
            config.file['filename'] = path.join(outputDir, fileName);
            fs.ensureDirSync(path.dirname(config.file['filename']));
            options.transports.push(new Winston.transports.File(config.file));
        }
        return new Winston.Logger(options);
    }
};
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", configuration_1.Configuration)
], Logger.prototype, "config", void 0);
Logger = __decorate([
    typescript_ioc_1.Singleton,
    typescript_ioc_1.AutoWired,
    __metadata("design:paramtypes", [])
], Logger);
exports.Logger = Logger;
//# sourceMappingURL=logger.js.map