'use strict';
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const auth = require("passport");
const typescript_ioc_1 = require("typescript-ioc");
const logger_1 = require("../../logger");
const middleware_loader_1 = require("../../utils/middleware-loader");
const Groups = require("../group");
const request_1 = require("../stats/request");
let ApiAuth = class ApiAuth {
    authentication(apiRouter, apiKey, api, pipelineConfig) {
        const path = api.path;
        const authentications = this.sortMiddlewares(api.authentication, path);
        authentications.forEach((authentication, index) => {
            try {
                authentication = this.resolveReferences(authentication, pipelineConfig);
                const authStrategy = this.middlewareLoader.loadMiddleware('authentication/strategy', authentication.strategy);
                if (!authStrategy) {
                    this.logger.error('Error configuring authenticator. Invalid Strategy');
                }
                else {
                    auth.use(`${apiKey}_${index}`, authStrategy);
                    const authenticator = auth.authenticate(`${apiKey}_${index}`, { session: false, failWithError: true });
                    if (authentication.group) {
                        this.createAuthenticatorForGroup(apiRouter, api, authentication, authenticator);
                    }
                    else {
                        this.createAuthenticator(apiRouter, api, authentication, authenticator);
                    }
                    if (this.logger.isDebugEnabled) {
                        this.logger.debug(`Authentication Strategy [${this.middlewareLoader.getId(authentication.strategy)}] configured for path [${path}]`);
                    }
                }
            }
            catch (e) {
                this.logger.error(`Error configuring Authentication Strategy [${this.middlewareLoader.getId(authentication.strategy)}] for path [${path}]`, e);
            }
        });
    }
    resolveReferences(authentication, pipelineConfig) {
        if (authentication.use && pipelineConfig.authentication) {
            if (pipelineConfig.authentication[authentication.use]) {
                authentication = _.defaults(authentication, pipelineConfig.authentication[authentication.use]);
            }
            else {
                throw new Error(`Invalid reference ${authentication.use}. There is no configuration for this id.`);
            }
        }
        return authentication;
    }
    createAuthenticator(apiRouter, api, authentication, authenticator) {
        if (this.requestLogger.isRequestLogEnabled(api)) {
            apiRouter.use((req, res, next) => {
                authenticator(req, res, (err) => {
                    const requestLog = this.requestLogger.getRequestLog(req);
                    if (err) {
                        if (requestLog) {
                            requestLog.authentication = 'fail';
                        }
                        next(err);
                    }
                    else {
                        if (requestLog) {
                            requestLog.authentication = 'success';
                        }
                        next();
                    }
                });
            });
        }
        else {
            apiRouter.use(authenticator);
        }
    }
    createAuthenticatorForGroup(apiRouter, api, authentication, authenticator) {
        if (this.logger.isDebugEnabled()) {
            const groups = Groups.filter(api.group, authentication.group);
            this.logger.debug(`Configuring Group filters for Authentication on path [${api.path}]. Groups [${JSON.stringify(groups)}]`);
        }
        const f = Groups.buildGroupAllowFilter(api.group, authentication.group);
        if (this.requestLogger.isRequestLogEnabled(api)) {
            apiRouter.use((req, res, next) => {
                if (f(req, res)) {
                    authenticator(req, res, (err) => {
                        const requestLog = this.requestLogger.getRequestLog(req);
                        if (err) {
                            if (requestLog) {
                                requestLog.authentication = 'fail';
                            }
                            next(err);
                        }
                        else {
                            if (requestLog) {
                                requestLog.authentication = 'success';
                            }
                            next();
                        }
                    });
                }
                else {
                    next();
                }
            });
        }
        else {
            apiRouter.use((req, res, next) => {
                if (f(req, res)) {
                    authenticator(req, res, next);
                }
                else {
                    next();
                }
            });
        }
    }
    sortMiddlewares(middlewares, path) {
        const generalMiddlewares = _.filter(middlewares, (value) => {
            if (value.group) {
                return false;
            }
            return true;
        });
        if (generalMiddlewares.length > 1) {
            this.logger.error(`Invalid authentication configuration for api [${path}]. Conflicting configurations for default group`);
            return [];
        }
        if (generalMiddlewares.length > 0) {
            const index = middlewares.indexOf(generalMiddlewares[0]);
            if (index < middlewares.length - 1) {
                const gen = middlewares.splice(index, 1);
                middlewares.push(gen[0]);
            }
        }
        return middlewares;
    }
};
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", logger_1.Logger)
], ApiAuth.prototype, "logger", void 0);
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", middleware_loader_1.MiddlewareLoader)
], ApiAuth.prototype, "middlewareLoader", void 0);
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", request_1.RequestLogger)
], ApiAuth.prototype, "requestLogger", void 0);
ApiAuth = __decorate([
    typescript_ioc_1.AutoWired
], ApiAuth);
exports.ApiAuth = ApiAuth;
//# sourceMappingURL=auth.js.map