'use strict';
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_ioc_1 = require("typescript-ioc");
const database_1 = require("../../database");
const logger_1 = require("../../logger");
const time_intervals_1 = require("../../utils/time-intervals");
const redis_store_1 = require("./redis-store");
class ServerCache {
    constructor() {
        if (!ServerCache.cacheStore) {
            this.initializeCacheStore();
        }
    }
    buildCacheMiddleware(serverCache, path, req, res, next, requestLog) {
        if (this.logger.isDebugEnabled()) {
            this.logger.debug('Configuring Server Cache for path [%s].', path);
        }
        const result = new Array();
        result.push(`ServerCache.cacheStore.get(${req}.originalUrl, function(err, entry){`);
        result.push(`if (err) {`);
        if (requestLog) {
            result.push(`${req}.requestLog.cache = 'error';`);
        }
        result.push(`return ${next}();`);
        result.push('}');
        result.push('if (entry) {');
        // cache hit
        if (requestLog) {
            result.push(`${req}.requestLog.cache = 'hit';`);
        }
        result.push(`${res}.contentType(entry.mimeType || "text/html");`);
        if (serverCache.preserveHeaders) {
            serverCache.preserveHeaders.forEach((header) => {
                result.push(`${res}.set('${header}', entry.header['${header}']);`);
            });
        }
        result.push(`${res}.send(new Buffer(entry.content, "base64"));`);
        result.push(`}`);
        result.push(`else {`);
        // cache miss
        if (requestLog) {
            result.push(`${req}.requestLog.cache = 'miss';`);
        }
        result.push(`${req}.parseRespBody = true;`);
        result.push(`var send = ${res}.send.bind(${res});`);
        result.push(`${res}.send = function (body) {`);
        result.push(`var ret = send(body);`);
        result.push(`if ( !body ) {`);
        result.push(`body = '';`);
        result.push(`}`);
        result.push(`body = new Buffer(body).toString("base64");`);
        result.push(`if ( typeof body !== "string" ) {`);
        result.push(`return ret;`);
        result.push(`}`);
        result.push(`if ((${res}.statusCode >= 200 && ${res}.statusCode < 300) || (${res}.statusCode === 304)){`);
        const cacheTime = time_intervals_1.getMilisecondsInterval(serverCache.cacheTime);
        result.push(`ServerCache.cacheStore.set(${req}.originalUrl, {`);
        result.push(`content: body,`);
        result.push(`mimeType: this._headers["content-type"]`);
        if (serverCache.preserveHeaders) {
            result.push(',header: {');
            serverCache.preserveHeaders.forEach((header, index) => {
                if (index > 0) {
                    result.push(`,`);
                }
                result.push(`'${header}': this._headers['${header}']`);
            });
            result.push(`}`);
        }
        result.push(`}, ${cacheTime});`);
        result.push(`}`);
        result.push(`return ret;`);
        result.push(`};`);
        result.push(`return ${next}();`);
        // end cache miss
        result.push(`}`);
        result.push(`});`);
        result.push(`return;`);
        return result.join('');
    }
    initializeCacheStore() {
        if (this.logger.isDebugEnabled()) {
            this.logger.debug('Initializing Redis cache store.');
        }
        ServerCache.cacheStore = new redis_store_1.RedisStore({
            client: this.database.redisClient
        });
    }
}
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", logger_1.Logger)
], ServerCache.prototype, "logger", void 0);
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", database_1.Database)
], ServerCache.prototype, "database", void 0);
exports.ServerCache = ServerCache;
//# sourceMappingURL=server-cache.js.map