'use strict';
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_ioc_1 = require("typescript-ioc");
const configuration_1 = require("../../configuration");
const logger_1 = require("../../logger");
const middleware_loader_1 = require("../../utils/middleware-loader");
const request_1 = require("../stats/request");
/**
 * The API Error handler. Called to handle errors in API pipeline processing.
 */
let ApiErrorHandler = class ApiErrorHandler {
    /**
     * Configure a proxy for a given API
     */
    handle(apiRouter, api, pipelineConfig) {
        apiRouter.use(this.configureErrorHandler(api, pipelineConfig));
    }
    configureErrorHandler(api, pipelineConfig) {
        const apiErrorHandler = api.errorHandler ? this.resolveReferences(api.errorHandler, pipelineConfig) : null;
        const errorHandler = apiErrorHandler ? apiErrorHandler.middleware : this.config.gateway.errorHandler;
        const appErrorHandler = this.getAppErrorHandler(errorHandler);
        if (this.requestLogger.isRequestLogEnabled(api)) {
            return (err, req, res, next) => {
                if (err && err.message) {
                    const requestLog = this.requestLogger.getRequestLog(req);
                    if (requestLog) {
                        requestLog.error = err.message;
                    }
                }
                appErrorHandler(err, req, res, next);
            };
        }
        else {
            return appErrorHandler;
        }
    }
    resolveReferences(errorHandler, pipelineConfig) {
        if (errorHandler.use && pipelineConfig.errorHandler) {
            if (pipelineConfig.errorHandler[errorHandler.use]) {
                errorHandler.middleware = pipelineConfig.errorHandler[errorHandler.use];
            }
            else {
                throw new Error(`Invalid reference ${errorHandler.use}. There is no configuration for this id.`);
            }
        }
        return errorHandler;
    }
    getAppErrorHandler(errorHandler) {
        if (errorHandler) {
            return this.middlewareLoader.loadMiddleware('errorhandler', errorHandler);
        }
        else {
            return (err, req, res, next) => {
                if (err && err.message) {
                    if (res.headersSent) { // important to allow default error handler to close connection if headers already sent
                        return next(err);
                    }
                    const mime = req.accepts('json', 'xml', 'html', 'text');
                    res.status(err.statusCode || err.status || 500);
                    switch (mime) {
                        case 'json':
                            res.set('Content-Type', 'application/json');
                            res.json({ error: err.message });
                            break;
                        case 'xml':
                            res.set('Content-Type', 'application/xml');
                            res.send(`<error>${err.message}</error>`);
                            break;
                        case 'html':
                            res.set('Content-Type', 'text/html');
                            res.send(`<html><head></head><body>${err.message}</body></html>`);
                            break;
                        default:
                            res.set('Content-Type', 'text/plain');
                            res.send(err.message);
                    }
                    if (this.logger.isWarnEnabled()) {
                        this.logger.warn(`Error processing API pipeline: ${err.message}`);
                        this.logger.warn(err);
                    }
                }
                else {
                    next(err);
                }
            };
        }
    }
};
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", configuration_1.Configuration)
], ApiErrorHandler.prototype, "config", void 0);
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", logger_1.Logger)
], ApiErrorHandler.prototype, "logger", void 0);
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", middleware_loader_1.MiddlewareLoader)
], ApiErrorHandler.prototype, "middlewareLoader", void 0);
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", request_1.RequestLogger)
], ApiErrorHandler.prototype, "requestLogger", void 0);
ApiErrorHandler = __decorate([
    typescript_ioc_1.AutoWired
], ApiErrorHandler);
exports.ApiErrorHandler = ApiErrorHandler;
//# sourceMappingURL=error-handler.js.map