'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const Joi = require("joi");
const mustache = require("mustache");
const typescript_ioc_1 = require("typescript-ioc");
const errors_1 = require("../../../config/errors");
const logger_1 = require("../../../logger");
const mustacheConfigSchema = Joi.object().keys({
    contentType: Joi.string(),
    template: Joi.string().required()
});
function validateMustacheConfig(config) {
    const result = Joi.validate(config, mustacheConfigSchema);
    if (result.error) {
        throw new errors_1.ValidationError(result.error);
    }
    else {
        return result.value;
    }
}
module.exports = function (config) {
    validateMustacheConfig(config);
    const template = config.template;
    const logger = typescript_ioc_1.Container.get(logger_1.Logger);
    return (err, req, res, next) => {
        if (err && err.message) {
            if (res.headersSent) { // important to allow default error handler to close connection if headers already sent
                return next(err);
            }
            res.set('Content-Type', config.contentType || 'text/html');
            res.status(err.statusCode || err.status || 500);
            let body;
            try {
                body = mustache.render(template, {
                    error: err,
                    req: req,
                    res: res
                });
            }
            catch (e) {
                body = { error: err.message };
            }
            res.send(body);
            if (logger.isWarnEnabled()) {
                logger.warn(`Error on API pipeline processing: ${err.message}`);
            }
        }
        else {
            next(err);
        }
    };
};
module.exports.factory = true;
//# sourceMappingURL=mustache.js.map