'use strict';
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_ioc_1 = require("typescript-ioc");
const logger_1 = require("../../logger");
const middleware_loader_1 = require("../../utils/middleware-loader");
const errors_1 = require("../error/errors");
const Groups = require("../group");
class ApiFilter {
    buildApiFilters(apiRouter, api, pipelineConfig) {
        if (api.proxy.target.allow) {
            this.buildAllowFilter(apiRouter, api);
        }
        if (api.proxy.target.deny) {
            this.buildDenyFilter(apiRouter, api);
        }
        if (this.hasCustomFilter(api)) {
            this.buildCustomFilters(apiRouter, api, pipelineConfig);
        }
    }
    buildGatewayFilters(apiRouter, filters) {
        if (filters) {
            if (this.logger.isDebugEnabled()) {
                this.logger.debug(`Configuring custom filters for Tree Gateway. Filters [${JSON.stringify(filters)}]`);
            }
            filters.forEach((filter) => {
                const filterMiddleware = this.middlewareLoader.loadMiddleware('filter', filter);
                apiRouter.use((req, res, next) => {
                    Promise.resolve(filterMiddleware(req, res)).then(result => {
                        if (result) {
                            return next();
                        }
                        if (!res.headersSent) {
                            next(new errors_1.NotFoundError());
                            // res.sendStatus(404);
                        }
                    }).catch(err => {
                        next(err);
                    });
                });
            });
        }
    }
    buildCustomFilters(apiRouter, api, pipelineConfig) {
        if (this.logger.isDebugEnabled()) {
            this.logger.debug(`Configuring custom filters for Proxy target [${api.path}]. Filters [${JSON.stringify(api.filter)}]`);
        }
        api.filter.forEach((filter) => {
            filter = this.resolveReferences(filter, pipelineConfig);
            const filterMiddleware = this.middlewareLoader.loadMiddleware('filter', filter.middleware);
            if (filter.group) {
                const groupValidator = Groups.buildGroupAllowFilter(api.group, filter.group);
                apiRouter.use((req, res, next) => {
                    if (groupValidator(req, res)) {
                        Promise.resolve(filterMiddleware(req, res)).then(result => {
                            if (result) {
                                return next();
                            }
                            if (!res.headersSent) {
                                next(new errors_1.NotFoundError());
                                // res.sendStatus(404);
                            }
                        }).catch(err => {
                            next(err);
                        });
                    }
                    else {
                        next();
                    }
                });
            }
            else {
                apiRouter.use((req, res, next) => {
                    Promise.resolve(filterMiddleware(req, res)).then(result => {
                        if (result) {
                            return next();
                        }
                        if (!res.headersSent) {
                            next(new errors_1.NotFoundError());
                            // res.sendStatus(404);
                        }
                    }).catch(err => {
                        next(err);
                    });
                });
            }
        });
    }
    resolveReferences(filter, pipelineConfig) {
        if (filter.use && pipelineConfig.filter) {
            if (pipelineConfig.filter[filter.use]) {
                filter.middleware = pipelineConfig.filter[filter.use];
            }
            else {
                throw new Error(`Invalid reference ${filter.use}. There is no configuration for this id.`);
            }
        }
        return filter;
    }
    buildAllowFilter(apiRouter, api) {
        if (this.logger.isDebugEnabled()) {
            const groups = Groups.filter(api.group, api.proxy.target.allow);
            this.logger.debug(`Configuring allow filter for Proxy target [${api.path}]. Groups [${JSON.stringify(groups)}]`);
        }
        const groupValidator = Groups.buildGroupAllowFilter(api.group, api.proxy.target.allow);
        apiRouter.use((req, res, next) => {
            if (groupValidator(req, res)) {
                next();
            }
            else {
                next(new errors_1.NotFoundError());
                // res.sendStatus(404);
            }
        });
    }
    buildDenyFilter(apiRouter, api) {
        if (this.logger.isDebugEnabled()) {
            const groups = Groups.filter(api.group, api.proxy.target.deny);
            this.logger.debug(`Configuring deny filter for Proxy target [${api.path}]. Groups [${JSON.stringify(groups)}]`);
        }
        const groupValidator = Groups.buildGroupDenyFilter(api.group, api.proxy.target.deny);
        apiRouter.use((req, res, next) => {
            if (groupValidator(req, res)) {
                next();
            }
            else {
                next(new errors_1.NotFoundError());
                // res.sendStatus(404);
            }
        });
    }
    hasCustomFilter(api) {
        return (api.filter && api.filter.length > 0);
    }
}
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", middleware_loader_1.MiddlewareLoader)
], ApiFilter.prototype, "middlewareLoader", void 0);
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", logger_1.Logger)
], ApiFilter.prototype, "logger", void 0);
exports.ApiFilter = ApiFilter;
//# sourceMappingURL=filter.js.map