'use strict';
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const mm = require("micromatch");
const typescript_ioc_1 = require("typescript-ioc");
const logger_1 = require("../../logger");
const functions_1 = require("../../utils/functions");
const middleware_loader_1 = require("../../utils/middleware-loader");
const Groups = require("../group");
class ProxyInterceptor {
    buildRequestInterceptors(apiRouter, api, pipelineConfig) {
        if (this.hasRequestInterceptor(api)) {
            this.createRequestInterceptors(apiRouter, api, pipelineConfig);
        }
    }
    buildResponseInterceptors(api, pipelineConfig) {
        if (this.hasResponseInterceptor(api)) {
            return (this.createResponseInterceptors(api, pipelineConfig));
        }
        return null;
    }
    hasRequestInterceptor(api) {
        return (api.interceptor && api.interceptor.request && api.interceptor.request.length > 0);
    }
    hasResponseInterceptor(api) {
        return (api.interceptor && api.interceptor.response && api.interceptor.response.length > 0);
    }
    resolveRequestInterceptorReferences(interceptor, pipelineConfig) {
        if (interceptor.use && pipelineConfig.interceptor && pipelineConfig.interceptor.request) {
            if (pipelineConfig.interceptor.request[interceptor.use]) {
                interceptor.middleware = pipelineConfig.interceptor.request[interceptor.use];
            }
            else {
                throw new Error(`Invalid reference ${interceptor.use}. There is no configuration for this id.`);
            }
        }
        return interceptor;
    }
    resolveResponseInterceptorReferences(interceptor, pipelineConfig) {
        if (interceptor.use && pipelineConfig.interceptor && pipelineConfig.interceptor.response) {
            if (pipelineConfig.interceptor.response[interceptor.use]) {
                interceptor.middleware = pipelineConfig.interceptor.response[interceptor.use];
            }
            else {
                throw new Error(`Invalid reference ${interceptor.use}. There is no configuration for this id.`);
            }
        }
        return interceptor;
    }
    createRequestInterceptors(apiRouter, api, pipelineConfig) {
        if (this.logger.isDebugEnabled()) {
            this.logger.debug(`Configuring request interceptors for Proxy target [${api.path}]. Interceptors [${JSON.stringify(api.interceptor.request)}]`);
        }
        api.interceptor.request.forEach((interceptor, index) => {
            interceptor = this.resolveRequestInterceptorReferences(interceptor, pipelineConfig);
            const interceptorMiddleware = this.middlewareLoader.loadMiddleware('interceptor/request', interceptor.middleware);
            if (interceptor.group) {
                const groupValidator = Groups.buildGroupAllowFilter(api.group, interceptor.group);
                apiRouter.use((req, res, next) => {
                    if (groupValidator(req, res)) {
                        const proxyReq = req.proxyReq || {
                            body: req.body,
                            connection: req.connection,
                            headers: Object.assign({}, req.headers),
                            method: req.method,
                            url: req.url,
                            user: req.user
                        };
                        Promise.resolve(interceptorMiddleware(proxyReq))
                            .then(result => {
                            req.proxyReq = Object.assign(proxyReq, result || {});
                            next();
                        }).catch(err => {
                            next(err);
                        });
                    }
                    else {
                        next();
                    }
                });
            }
            else {
                apiRouter.use((req, res, next) => {
                    const proxyReq = req.proxyReq || {
                        body: req.body,
                        connection: req.connection,
                        headers: Object.assign({}, req.headers),
                        method: req.method,
                        url: req.url,
                        user: req.user
                    };
                    Promise.resolve(interceptorMiddleware(proxyReq))
                        .then(result => {
                        req.proxyReq = Object.assign(proxyReq, result || {});
                        next();
                    }).catch(err => {
                        next(err);
                    });
                });
            }
        });
    }
    createResponseInterceptors(api, pipelineConfig) {
        const body = new Array();
        const interceptors = {};
        const result = {
            middelware: null,
            validators: []
        };
        body.push(`var continueChain = function(body, headers, request){ return {body: body}; };`);
        api.interceptor.response.forEach((interceptor, index) => {
            interceptor = this.resolveResponseInterceptorReferences(interceptor, pipelineConfig);
            const interceptorMiddleware = this.middlewareLoader.loadMiddleware('interceptor/response', interceptor.middleware);
            const middlewareId = this.middlewareLoader.getId(interceptor.middleware);
            interceptors[middlewareId] = interceptorMiddleware;
            if (interceptor.group) {
                result.validators.push(Groups.buildGroupNotAllowFilter(api.group, interceptor.group));
            }
            else {
                result.validators.push((req) => false);
            }
            body.push(`var f${index};`);
            body.push(`if (ignore[${index}])`);
            body.push(`f${index} = continueChain;`);
            body.push(`else f${index} = interceptors['${middlewareId}'];`);
            body.push(`Promise.resolve(f${index}(body, proxyRes.headers, request)).catch((error) => { \
                   callback(error); \
                   return; \
                }).then(result => { \
                    body = result.body; \
                    headersHandler(result.updateHeaders, result.removeHeaders);`);
        });
        api.interceptor.response.forEach((interceptor, index) => {
            if (index === 0) {
                body.push(`callback(null, body);`);
            }
            body.push(`});`);
        });
        result.middelware = functions_1.createFunction({
            interceptors: interceptors,
            mm: mm
        }, 'body', 'proxyRes', 'request', 'response', 'ignore', 'headersHandler', 'callback', body.join(''));
        return result;
    }
}
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", middleware_loader_1.MiddlewareLoader)
], ProxyInterceptor.prototype, "middlewareLoader", void 0);
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", logger_1.Logger)
], ProxyInterceptor.prototype, "logger", void 0);
exports.ProxyInterceptor = ProxyInterceptor;
//# sourceMappingURL=interceptor.js.map