'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const Joi = require("joi");
const _ = require("lodash");
const errors_1 = require("../../../config/errors");
const requestHeadersSchema = Joi.object().keys({
    headers: Joi.object().unknown(true),
    removeHeaders: Joi.alternatives([Joi.array().items(Joi.string()), Joi.string()]),
    updateHeaders: Joi.object().unknown(true)
});
function validateRequestHeadersConfig(config) {
    const result = Joi.validate(config, requestHeadersSchema);
    if (result.error) {
        throw new errors_1.ValidationError(result.error);
    }
    else {
        return result.value;
    }
}
module.exports = function (config) {
    validateRequestHeadersConfig(config);
    if (config.removeHeaders) {
        config.removeHeaders = _.castArray(config.removeHeaders).map(header => header.toLowerCase());
    }
    const updateHeaders = config.updateHeaders || config.headers;
    return (req) => {
        let h = req.headers;
        if (updateHeaders) {
            h = Object.assign(h, updateHeaders);
        }
        if (config.removeHeaders) {
            h = _.omit(h, ...config.removeHeaders);
        }
        return { headers: h };
    };
};
module.exports.factory = true;
//# sourceMappingURL=requestHeaders.js.map