'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const Joi = require("joi");
const mustache = require("mustache");
const errors_1 = require("../../../config/errors");
const mustacheConfigSchema = Joi.object().keys({
    template: Joi.string().required(),
});
function validateMustacheConfig(config) {
    const result = Joi.validate(config, mustacheConfigSchema);
    if (result.error) {
        throw new errors_1.ValidationError(result.error);
    }
    else {
        return result.value;
    }
}
module.exports = function (config) {
    validateMustacheConfig(config);
    const template = config.template;
    return (body, headers, request) => {
        const result = {};
        try {
            if (body && (typeof body === 'string') || Buffer.isBuffer(body)) {
                body = JSON.parse(body.toString());
            }
            result.body = mustache.render(template, body || {});
        }
        catch (e) {
            result.body = '';
        }
        return result;
    };
};
module.exports.factory = true;
//# sourceMappingURL=responseMustache.js.map