'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const Joi = require("joi");
const _ = require("lodash");
const errors_1 = require("../../../config/errors");
const responseHeaders = require('./responseHeaders');
const configSchema = Joi.object().keys({
    dnsPrefetchControl: Joi.boolean(),
    frameguard: Joi.string().valid('DENY', 'ALLOW-FROM', 'SAMEORIGIN'),
    frameguardDomain: Joi.string(),
    hidePoweredBy: Joi.boolean(),
    noSniff: Joi.boolean(),
    xssFilter: Joi.boolean()
});
function validateConfig(config) {
    const result = Joi.validate(config, configSchema);
    if (result.error) {
        throw new errors_1.ValidationError(result.error);
    }
    else {
        return result.value;
    }
}
module.exports = function (config) {
    config = _.defaults(config, {
        dnsPrefetchControl: true,
        frameguard: 'DENY',
        hidePoweredBy: true,
        noSniff: true,
        xssFilter: true
    });
    validateConfig(config);
    const options = {
        updateHeaders: {}
    };
    if (config.hidePoweredBy) {
        options.removeHeaders = 'X-Powered-By';
    }
    if (config.noSniff) {
        options.updateHeaders['X-Content-Type-Options'] = 'nosniff';
    }
    if (!config.dnsPrefetchControl) {
        options.updateHeaders['X-DNS-Prefetch-Control'] = 'off';
    }
    if (config.xssFilter) {
        options.updateHeaders['X-XSS-Protection'] = '1; mode=block';
    }
    if (config.frameguard) {
        let value = config.frameguard;
        if (value === 'ALLOW-FROM') {
            value = `${value} ${config.frameguardDomain}`;
        }
        options.updateHeaders['X-Frame-Options'] = value;
    }
    return responseHeaders(options);
};
module.exports.factory = true;
//# sourceMappingURL=webSecurity.js.map