'use strict';
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const stream_buffers_1 = require("stream-buffers");
const typescript_ioc_1 = require("typescript-ioc");
const url = require("url");
const logger_1 = require("../../logger");
const middleware_loader_1 = require("../../utils/middleware-loader");
const time_intervals_1 = require("../../utils/time-intervals");
const service_discovery_1 = require("../servicediscovery/service-discovery");
const request_1 = require("../stats/request");
const interceptor_1 = require("./interceptor");
const agentKeepAlive = require('agentkeepalive');
const httpProxy = require('../../../lib/http-proxy');
/**
 * The API Proxy system. It uses [[http-proxy]](https://github.com/nodejitsu/node-http-proxy)
 * to proxy requests to a target API.
 */
let ApiProxy = class ApiProxy {
    /**
     * Configure a proxy for a given API
     */
    proxy(apiRouter, api, pipelineConfig) {
        this.buildRouterMiddleware(apiRouter, api);
        this.buildServiceDiscoveryMiddleware(apiRouter, api);
        this.interceptor.buildRequestInterceptors(apiRouter, api, pipelineConfig);
        apiRouter.use(this.configureProxy(api, pipelineConfig));
    }
    configureProxyHeader(apiRouter, api) {
        if (!api.proxy.supressViaHeader) {
            const onHeaders = require('on-headers');
            const gatewayId = 'Tree-Gateway';
            apiRouter.use((req, res, next) => {
                onHeaders(res, () => {
                    let viaHeader = res.get('Via');
                    if (viaHeader) {
                        viaHeader = viaHeader + ', ' + req.httpVersion + ' ' + gatewayId;
                    }
                    else {
                        viaHeader = req.httpVersion + ' ' + gatewayId;
                    }
                    res.set('Via', viaHeader);
                });
                next();
            });
        }
    }
    getHttpAgent(api) {
        const apiAgentOptions = api.proxy.httpAgent || {};
        const agentOptions = {
            keepAlive: true
        };
        if (_.isBoolean(apiAgentOptions.keepAlive) && !apiAgentOptions.keepAlive) {
            agentOptions.keepAlive = false;
        }
        if (agentOptions.keepAlive) {
            agentOptions.keepAliveMsecs = time_intervals_1.getMilisecondsInterval(apiAgentOptions.keepAliveTime, 1000);
            agentOptions.freeSocketKeepAliveTimeout = time_intervals_1.getMilisecondsInterval(apiAgentOptions.freeSocketKeepAliveTimeout, 15000);
            agentOptions.maxFreeSockets = apiAgentOptions.maxFreeSockets || 256;
        }
        if (apiAgentOptions.maxSockets) {
            agentOptions.maxSockets = apiAgentOptions.maxSockets;
        }
        if (apiAgentOptions.timeout) {
            agentOptions.timeout = time_intervals_1.getMilisecondsInterval(apiAgentOptions.timeout);
        }
        if (api.proxy.target.router) {
            if (api.proxy.target.router.ssl) {
                return new agentKeepAlive.HttpsAgent(agentOptions);
            }
        }
        else {
            const proxyTarget = url.parse(api.proxy.target.host);
            if (proxyTarget.protocol && proxyTarget.protocol === 'https:') {
                return new agentKeepAlive.HttpsAgent(agentOptions);
            }
        }
        return new agentKeepAlive(agentOptions);
    }
    configureProxy(api, pipelineConfig) {
        const apiProxy = api.proxy;
        const proxyConfig = {
            changeOrigin: !apiProxy.preserveHostHdr,
            target: apiProxy.target.host
        };
        const httpAgent = this.getHttpAgent(api);
        if (httpAgent) {
            proxyConfig.agent = httpAgent;
        }
        if (apiProxy.timeout) {
            proxyConfig.proxyTimeout = time_intervals_1.getMilisecondsInterval(apiProxy.timeout);
        }
        const proxy = httpProxy.createProxyServer(proxyConfig);
        const requestLogEnabled = this.requestLogger.isRequestLogEnabled(api);
        proxy.on('error', (err, req, res) => {
            if (requestLogEnabled) {
                const requestLog = this.requestLogger.getRequestLog(req);
                requestLog.proxyTime -= new Date().getTime();
            }
            const hostname = (req.headers && req.headers.host) || (req.hostname || req.host); // (websocket) || (node0.10 || node 4/5)
            const target = apiProxy.target.host;
            const errReference = 'https://nodejs.org/api/errors.html#errors_common_system_errors'; // link to Node Common Systems Errors page
            this.logger.error('[Tree-Gateway] Error occurred while trying to proxy request %s from %s to %s (%s) (%s)', req.url, hostname, target, err.code, errReference);
            if (!res.headersSent) {
                res.status(502).send('Bad Gateway');
            }
        });
        if (requestLogEnabled) {
            proxy.on('start', (req, res) => {
                const requestLog = this.requestLogger.getRequestLog(req);
                requestLog.proxyTime = new Date().getTime();
            });
        }
        const maybeWrapResponse = this.interceptor.hasResponseInterceptor(api);
        const responseInterceptor = this.interceptor.buildResponseInterceptors(api, pipelineConfig);
        this.handleResponseInterceptor(api, proxy, responseInterceptor);
        function validateInterceptors(req, res) {
            if (responseInterceptor) {
                let ignoreAll = true;
                res.__ignore = new Array();
                responseInterceptor.validators.forEach((validator, index) => {
                    const ignore = validator(req);
                    res.__ignore.push(ignore);
                    ignoreAll = ignoreAll && ignore;
                });
                return maybeWrapResponse && !ignoreAll;
            }
            return false;
        }
        return (req, res, next) => {
            if (validateInterceptors(req, res) || req.parseRespBody) {
                const options = req.proxyOptions || {};
                res.__data = new stream_buffers_1.WritableStreamBuffer();
                options.destPipe = { stream: res.__data };
                proxy.web(req, res, options);
            }
            else if (req.proxyOptions) {
                proxy.web(req, res, req.proxyOptions);
            }
            else {
                proxy.web(req, res);
            }
        };
    }
    handleResponseInterceptor(api, proxy, responseInterceptor) {
        const requestLogEnabled = this.requestLogger.isRequestLogEnabled(api);
        proxy.on('end', (req, res, proxyRes) => {
            if (requestLogEnabled) {
                const requestLog = this.requestLogger.getRequestLog(req);
                requestLog.proxyTime -= new Date().getTime();
            }
            if (res.__data) {
                if (responseInterceptor) {
                    responseInterceptor.middelware(res.__data.getContents(), proxyRes, req, res, res.__ignore, (newHeaders, removeHeaders) => {
                        if (newHeaders) {
                            Object.keys(newHeaders).forEach(name => {
                                proxyRes.headers[name.toLowerCase()] = newHeaders[name];
                                res.set(name.toLowerCase(), newHeaders[name]);
                            });
                        }
                        if (removeHeaders) {
                            removeHeaders.forEach(name => {
                                delete proxyRes.headers[name];
                                res.removeHeader(name);
                                if (name !== name.toLowerCase()) {
                                    delete proxyRes.headers[name.toLowerCase()];
                                    res.removeHeader(name.toLowerCase());
                                }
                            });
                        }
                    }, (err, body) => {
                        if (err) {
                            this.logger.error(err);
                        }
                        delete res['__data'];
                        if (!res.finished) {
                            res.send(body);
                        }
                    });
                }
                else {
                    const body = res.__data.getContents();
                    delete res['__data'];
                    if (!res.finished) {
                        res.send(body);
                    }
                }
            }
        });
    }
    buildRouterMiddleware(apiRouter, api) {
        if (api.proxy.target.router) {
            const router = api.proxy.target.router;
            if (api.proxy.target.router.middleware) {
                const routerMiddleware = this.middlewareLoader.loadMiddleware('proxy/router', router.middleware);
                apiRouter.use((req, res, next) => {
                    Promise.resolve(routerMiddleware(req))
                        .then(result => {
                        req.proxyOptions = { target: result };
                        next();
                    }).catch(err => {
                        next(err);
                    });
                });
            }
        }
    }
    buildServiceDiscoveryMiddleware(apiRouter, api) {
        const router = api.proxy.target.router;
        if (router) {
            if (router.serviceDiscovery) {
                const serviceDiscovery = this.serviceDiscovery.loadServiceDiscovery(router.serviceDiscovery, router.ssl);
                apiRouter.use((req, res, next) => {
                    Promise.resolve(serviceDiscovery(req.proxyOptions ? req.proxyOptions.target : null))
                        .then(result => {
                        req.proxyOptions = { target: result };
                        next();
                    }).catch(err => {
                        next(err);
                    });
                });
            }
        }
    }
};
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", interceptor_1.ProxyInterceptor)
], ApiProxy.prototype, "interceptor", void 0);
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", logger_1.Logger)
], ApiProxy.prototype, "logger", void 0);
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", middleware_loader_1.MiddlewareLoader)
], ApiProxy.prototype, "middlewareLoader", void 0);
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", service_discovery_1.ServiceDiscovery)
], ApiProxy.prototype, "serviceDiscovery", void 0);
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", request_1.RequestLogger)
], ApiProxy.prototype, "requestLogger", void 0);
ApiProxy = __decorate([
    typescript_ioc_1.AutoWired
], ApiProxy);
exports.ApiProxy = ApiProxy;
//# sourceMappingURL=proxy.js.map