'use strict';
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const typescript_ioc_1 = require("typescript-ioc");
const database_1 = require("../../database");
class RedisStore {
    constructor(options) {
        this.options = _.defaults(options, {
            expire: 60,
            prefix: 'rl'
        });
    }
    incr(key, cb) {
        const rdskey = this.getRedisKey(key);
        const opt = this.options;
        this.database.redisClient.multi()
            .incr(rdskey)
            .ttl(rdskey)
            .exec((err, replies) => {
            if (err) {
                return cb(err);
            }
            // if this is new or has no expire
            if (replies[0][1] === 1 || replies[1][1] === -1) {
                // then expire it after the timeout
                this.database.redisClient.expire(rdskey, opt.expire);
            }
            cb(null, replies[0][1]);
        });
    }
    resetKey(key) {
        const rdskey = this.getRedisKey(key);
        this.database.redisClient.del(rdskey);
    }
    getRedisKey(key) {
        const result = [];
        result.push(this.options.prefix);
        result.push(this.options.path);
        if (this.options.id) {
            result.push(this.options.id);
        }
        result.push(key);
        return result.join(':');
    }
}
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", database_1.Database)
], RedisStore.prototype, "database", void 0);
exports.RedisStore = RedisStore;
//# sourceMappingURL=redis-store.js.map