'use strict';
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_ioc_1 = require("typescript-ioc");
const api_1 = require("./api");
const gateway_1 = require("./gateway");
const middleware_1 = require("./middleware");
class ConfigPackageService {
}
exports.ConfigPackageService = ConfigPackageService;
let ConfigPackageServiceImpl = class ConfigPackageServiceImpl {
    /**
     * Apply a config package to this Gateway
     * @param config The config package to apply
     */
    async set(config) {
        if (config.middlewares) {
            await Promise.all(config.middlewares.map(mid => this.middlewareService.update(mid.middleware, mid.id || mid.name, new Buffer(mid.content), true)));
            await this.setApisAndGateway(config);
        }
        else {
            await this.setApisAndGateway(config);
        }
    }
    /**
     * Export all gateway configurations to a new config package.
     * @return The generated config package
     */
    async get() {
        const result = {};
        const data = await Promise.all([this.apiService.list(), this.gatewayService.get(), this.getMiddlewares()]);
        if (data && data.length) {
            if (data[0]) {
                result.apis = data[0];
            }
            if (data.length > 1 && data[1]) {
                result.gateway = data[1];
            }
            if (data.length > 2 && data[2]) {
                result.middlewares = data[2];
            }
        }
        return result;
    }
    async getMiddlewares() {
        const allMiddlewares = await Promise.all([
            this.middlewareService.list('filter'),
            this.middlewareService.list('interceptor/request'),
            this.middlewareService.list('interceptor/response'),
            this.middlewareService.list('authentication/strategy'),
            this.middlewareService.list('authentication/verify'),
            this.middlewareService.list('throttling/keyGenerator'),
            this.middlewareService.list('throttling/handler'),
            this.middlewareService.list('throttling/skip'),
            this.middlewareService.list('circuitbreaker'),
            this.middlewareService.list('cors/origin'),
            this.middlewareService.list('proxy/router'),
            this.middlewareService.list('servicediscovery'),
            this.middlewareService.list('servicediscovery/provider'),
            this.middlewareService.list('errorhandler'),
            this.middlewareService.list('request/logger')
        ]);
        const all = await Promise.all([
            this.getMiddlewaresByType('filter', allMiddlewares[0]),
            this.getMiddlewaresByType('interceptor/request', allMiddlewares[1]),
            this.getMiddlewaresByType('interceptor/response', allMiddlewares[2]),
            this.getMiddlewaresByType('authentication/strategy', allMiddlewares[3]),
            this.getMiddlewaresByType('authentication/verify', allMiddlewares[4]),
            this.getMiddlewaresByType('throttling/keyGenerator', allMiddlewares[5]),
            this.getMiddlewaresByType('throttling/handler', allMiddlewares[6]),
            this.getMiddlewaresByType('throttling/skip', allMiddlewares[7]),
            this.getMiddlewaresByType('circuitbreaker', allMiddlewares[8]),
            this.getMiddlewaresByType('cors/origin', allMiddlewares[9]),
            this.getMiddlewaresByType('proxy/router', allMiddlewares[10]),
            this.getMiddlewaresByType('servicediscovery', allMiddlewares[11]),
            this.getMiddlewaresByType('servicediscovery/provider', allMiddlewares[12]),
            this.getMiddlewaresByType('errorhandler', allMiddlewares[13]),
            this.getMiddlewaresByType('request/logger', allMiddlewares[14])
        ]);
        const result = [];
        all.forEach(middlewares => {
            middlewares.forEach(middleware => {
                result.push(middleware);
            });
        });
        return result;
    }
    async getMiddlewaresByType(middleware, names) {
        if (names && names.length) {
            const promises = names.map(name => new Promise((res, rej) => {
                this.middlewareService.read(middleware, name)
                    .then(content => res({ middleware: middleware, id: name, content: content.toString() }))
                    .catch(rej);
            }));
            return await Promise.all(promises);
        }
        else {
            return [];
        }
    }
    async setApisAndGateway(config) {
        let promises;
        if (config.apis) {
            promises = config.apis.map(api => this.apiService.update(api, true));
        }
        else {
            promises = [];
        }
        if (config.gateway) {
            promises.push(this.gatewayService.save(config.gateway));
        }
        await Promise.all(promises);
    }
};
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", api_1.ApiService)
], ConfigPackageServiceImpl.prototype, "apiService", void 0);
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", gateway_1.GatewayService)
], ConfigPackageServiceImpl.prototype, "gatewayService", void 0);
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", middleware_1.MiddlewareService)
], ConfigPackageServiceImpl.prototype, "middlewareService", void 0);
ConfigPackageServiceImpl = __decorate([
    typescript_ioc_1.AutoWired,
    typescript_ioc_1.Singleton,
    typescript_ioc_1.Provides(ConfigPackageService)
], ConfigPackageServiceImpl);
exports.ConfigPackageServiceImpl = ConfigPackageServiceImpl;
//# sourceMappingURL=config-package.js.map