'use strict';
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const bcrypt = require("bcryptjs");
const jwt = require("jsonwebtoken");
const _ = require("lodash");
const auth = require("passport");
const typescript_ioc_1 = require("typescript-ioc");
const errors_1 = require("../../config/errors");
const configuration_1 = require("../../configuration");
const database_1 = require("../../database");
const errors_2 = require("../../pipeline/error/errors");
const middleware_loader_1 = require("../../utils/middleware-loader");
class RedisUserService {
    async list() {
        const users = await this.database.redisClient.hgetall(RedisUserService.USERS_PREFIX);
        return _.map(_.values(users), (value) => _.omit(JSON.parse(value), 'password'));
    }
    async get(login) {
        const user = await this.database.redisClient.hget(RedisUserService.USERS_PREFIX, login);
        if (!user) {
            return null;
        }
        return JSON.parse(user);
    }
    async create(user) {
        const exists = await this.database.redisClient.hexists(`${RedisUserService.USERS_PREFIX}`, user.login);
        if (exists) {
            throw new errors_1.ValidationError(`User ${user.login} already exists`);
        }
        if (!user.password) {
            throw new errors_1.ValidationError(`Password is required for user.`);
        }
        user.password = await bcrypt.hash(user.password, 10);
        await this.database.redisClient.hmset(`${RedisUserService.USERS_PREFIX}`, user.login, JSON.stringify(user));
    }
    async changePassword(login, password) {
        const exists = await this.database.redisClient.hexists(`${RedisUserService.USERS_PREFIX}`, login);
        if (!exists) {
            throw new errors_2.NotFoundError('User not found.');
        }
        const user = await this.get(login);
        user.password = await bcrypt.hash(user.password, 10);
        await this.database.redisClient.hmset(`${RedisUserService.USERS_PREFIX}`, user.login, JSON.stringify(user));
    }
    async update(user) {
        const exists = await this.database.redisClient.hexists(`${RedisUserService.USERS_PREFIX}`, user.login);
        if (!exists) {
            throw new errors_2.NotFoundError('User not found.');
        }
        const oldUser = await this.get(user.login);
        user.password = oldUser.password;
        await this.database.redisClient.hmset(`${RedisUserService.USERS_PREFIX}`, user.login, JSON.stringify(user));
    }
    async remove(login) {
        const count = await this.database.redisClient.hdel(`${RedisUserService.USERS_PREFIX}`, login);
        if (count === 0) {
            throw new errors_2.NotFoundError('User not found.');
        }
    }
    async generateToken(login, password) {
        const user = await this.get(login);
        if (!user) {
            throw new errors_1.ValidationError('User not found');
        }
        let same;
        try {
            same = await bcrypt.compare(password, user.password);
        }
        catch (err) {
            throw new errors_1.ValidationError('Error validating user password');
        }
        if (!same) {
            throw new errors_1.ValidationError('Invalid username / password');
        }
        try {
            const dataToken = {
                email: user.email,
                login: user.login,
                name: user.name,
                roles: user.roles
            };
            const token = jwt.sign(dataToken, this.config.gateway.admin.userService.jwtSecret, {
                expiresIn: 7200 // TODO read an human interval configuration
            });
            return token;
        }
        catch (err) {
            throw new errors_1.ValidationError('Error validating user password');
        }
    }
    getAuthMiddleware() {
        const opts = {
            secretOrKey: this.config.gateway.admin.userService.jwtSecret
        };
        const strategy = this.middlewareLoader.loadMiddleware('authentication/strategy', { name: 'jwt', options: opts });
        auth.use('_tree_gateway_admin_', strategy);
        return auth.authenticate('_tree_gateway_admin_', { session: false, failWithError: true });
    }
}
RedisUserService.USERS_PREFIX = 'adminUsers';
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", configuration_1.Configuration)
], RedisUserService.prototype, "config", void 0);
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", database_1.Database)
], RedisUserService.prototype, "database", void 0);
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", middleware_loader_1.MiddlewareLoader)
], RedisUserService.prototype, "middlewareLoader", void 0);
exports.RedisUserService = RedisUserService;
//# sourceMappingURL=users.js.map