'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const jwt = require("jsonwebtoken");
const _ = require("lodash");
const path = require("path");
function getSwaggerHost(gateway) {
    let host;
    if (gateway.admin.apiDocs.host) {
        host = gateway.admin.apiDocs.host;
    }
    else {
        host = (gateway.admin.protocol.https ?
            `localhost:${gateway.admin.protocol.https.listenPort}` :
            `localhost:${gateway.admin.protocol.http.listenPort}`);
    }
    return host;
}
exports.getSwaggerHost = getSwaggerHost;
function getSwaggerUrl(gateway) {
    if (gateway && gateway.admin && gateway.admin.apiDocs) {
        const protocol = (gateway.admin.protocol.https ? 'https' : 'http');
        return `${protocol}://` + path.posix.join(`${getSwaggerHost(gateway)}`, gateway.admin.apiDocs.path, 'json');
    }
    throw new Error('No admin apiDocs configured. Can not access the server rest API');
}
exports.getSwaggerUrl = getSwaggerUrl;
function generateSecurityToken(gateway) {
    const dataToken = {
        login: 'treeGateway CLI',
        name: 'treeGateway CLI',
        roles: ['admin', 'config']
    };
    const token = jwt.sign(dataToken, gateway.admin.userService.jwtSecret, {
        expiresIn: 7200
    });
    return token;
}
exports.generateSecurityToken = generateSecurityToken;
function castArray(obj, objPath) {
    if (!obj || !objPath) {
        return;
    }
    if (_.has(obj, objPath)) {
        const value = _.get(obj, objPath, null);
        _.set(obj, objPath, _.castArray(value));
    }
}
exports.castArray = castArray;
//# sourceMappingURL=config.js.map