'use strict';
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs-extra-promise");
const path = require("path");
const typescript_ioc_1 = require("typescript-ioc");
const configuration_1 = require("../configuration");
const logger_1 = require("../logger");
const middleware_1 = require("../service/middleware");
const decache = require('decache');
let MiddlewareInstaller = class MiddlewareInstaller {
    constructor() {
        this.types = ['filter', 'interceptor/request', 'interceptor/response',
            'authentication/strategy', 'authentication/verify',
            'throttling/keyGenerator', 'throttling/handler',
            'throttling/skip', 'circuitbreaker', 'cors/origin', 'proxy/router',
            'servicediscovery', 'servicediscovery/provider', 'errorhandler', 'request/logger'];
    }
    async installAll() {
        await Promise.all(this.types.map(type => this.installAllOfType(type)));
    }
    async install(type, name) {
        await this.uninstall(type, name);
        if (this.logger.isDebugEnabled()) {
            this.logger.debug(`Installing middleware ${type}/${name}`);
        }
        const content = await this.service.read(type, name);
        await this.saveFile(this.getPath(type, name), content);
        if (this.logger.isDebugEnabled()) {
            this.logger.debug(`Middleware ${type}/${name} installed.`);
        }
    }
    async uninstall(type, name) {
        if (this.logger.isDebugEnabled()) {
            this.logger.debug(`Uninstalling middleware ${type}/${name}`);
        }
        const p = this.getPath(type, name);
        if (fs.existsSync(p)) {
            this.removeModuleCache(type, name);
            await fs.removeAsync(p);
        }
    }
    removeModuleCache(type, name) {
        decache(this.getModuleName(type, name));
    }
    async installAllOfType(type) {
        const names = await this.service.list(type);
        await Promise.all(names.map((name) => this.install(type, name)));
    }
    async saveFile(filePath, content) {
        // TODO: support zip files
        await fs.outputFileAsync(filePath, content);
    }
    getPath(type, name) {
        return this.getModuleName(type, name) + '.js';
    }
    getModuleName(type, name) {
        return path.join(this.config.middlewarePath, type, name);
    }
};
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", middleware_1.MiddlewareService)
], MiddlewareInstaller.prototype, "service", void 0);
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", logger_1.Logger)
], MiddlewareInstaller.prototype, "logger", void 0);
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", configuration_1.Configuration)
], MiddlewareInstaller.prototype, "config", void 0);
MiddlewareInstaller = __decorate([
    typescript_ioc_1.AutoWired,
    typescript_ioc_1.Singleton
], MiddlewareInstaller);
exports.MiddlewareInstaller = MiddlewareInstaller;
//# sourceMappingURL=middleware-installer.js.map