'use strict';
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = require("chalk");
const _ = require("lodash");
const typescript_ioc_1 = require("typescript-ioc");
const database_1 = require("../database");
const config_package_1 = require("../service/config-package");
const compareVersions = require('compare-versions');
let VersionUpgrades = class VersionUpgrades {
    async checkGatewayVersion() {
        const packageJson = require('../../package.json');
        const version = await this.database.getGatewayVersion();
        const compared = compareVersions(packageJson.version, version || '0.0.1');
        if (compared === 0) {
            return;
        }
        else if (compared < 0) {
            throw new Error('Can not load configuration. This configuration repository contains information generated by a newer tree-gateway version. Try to update tree-gateway, running npm i -g tree-gateway.');
        }
        await this.updateGatewayConfiguration();
    }
    async updateGatewayConfiguration() {
        const previous = await this.database.startGatewayUpdate();
        if (previous !== database_1.Database.UPDATING) {
            await this.update(previous);
        }
        else {
            await this.wait(0);
        }
    }
    async update(version) {
        if (compareVersions(version || '0.0.1', '3.0.0') < 0) {
            const pac = await this.service.get();
            if (pac && pac.gateway) {
                // tslint:disable-next-line:no-console
                console.info(chalk_1.default.magenta('An older configuration was found. Updating it to the newer format. '
                    + 'Check tree-gateway migration guide for more info.'));
                if (!pac.gateway['disableStats']) {
                    pac.gateway.analytics = { enabled: true, logger: { name: 'redis' } };
                }
                pac.gateway = _.omit(pac.gateway, 'statsConfig', 'monitor', 'disableStats');
            }
            if (pac.apis) {
                pac.apis = pac.apis.map(api => {
                    if (_.has(api, 'proxy.disableStats')) {
                        api.disableAnalytics = _.get(api.proxy, 'disableStats');
                    }
                    if (_.has(api, 'errorHandler')) {
                        api.errorHandler = { middleware: _.get(api.proxy, 'errorHandler') };
                    }
                    if (_.has(api, 'proxy.parseReqBody')) {
                        api.parseReqBody = _.get(api.proxy, 'parseReqBody');
                    }
                    if (_.has(api, 'proxy.parseCookies')) {
                        api.parseCookies = _.get(api.proxy, 'parseCookies');
                    }
                    if (_.has(api, 'proxy.interceptor')) {
                        api.interceptor = _.get(api.proxy, 'interceptor');
                    }
                    return _.omit(api, 'proxy.statsConfig', 'proxy.disableStats', 'proxy.parseReqBody', 'proxy.parseCookies', 'proxy.interceptor');
                });
            }
            await this.service.set(pac);
            await this.database.registerGatewayVersion();
        }
    }
    wait(count) {
        return new Promise((resolve, reject) => {
            if (count > 10) {
                return reject(new Error('Can not upgrade gateway configuration.'));
            }
            setTimeout(() => {
                this.database.getGatewayVersion()
                    .then(version => {
                    if (version === database_1.Database.UPDATING) {
                        this.wait(count++)
                            .then(resolve)
                            .catch(reject);
                    }
                    else {
                        resolve();
                    }
                });
            }, 100);
        });
    }
};
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", database_1.Database)
], VersionUpgrades.prototype, "database", void 0);
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", config_package_1.ConfigPackageService)
], VersionUpgrades.prototype, "service", void 0);
VersionUpgrades = __decorate([
    typescript_ioc_1.Singleton,
    typescript_ioc_1.AutoWired
], VersionUpgrades);
exports.VersionUpgrades = VersionUpgrades;
//# sourceMappingURL=upgrade.js.map